pipeline
{
	agent 
	{
		label 'jenkins-node-java'
	}
	parameters
	{
		credentials(name:'git_credential',defaultValue:'',description: '')
		string(name:'TagName',defaultValue:'',description: '')
	}
	stages
	{
		stage('Prepare')
		{
			steps
			{
				script
				{
					container('jenkins-node-java')
					{
						echo "Prepare start"
						sh 'rm -rf *'
						//参数定义
						blob_container = "containername"  //blob container名称
						webAppVersion=''
						dataWorkerVersion=''
						OTAWorkerVersion=''
						sourceCodeGitUrl="sourceCodeGiturl"  //填写源码gitlab地址
						blob_container_key = "containerkey"  //contain key
					}
				}
			}
		}
		stage("RMMServer_Build"){
			steps
			{
				script
				{
					container('jenkins-node-java')
					{
						stageIndex=45
						dir('RMMServer')
						{
							retry(3)
							{
								git credentialsId: "${git_credential}", url: "${sourceCodeGitUrl}.git"
							}
							if(TagName=="")
							{
								//获取最新的tag的名字
								sh 'git describe --tags `git rev-list --tags --max-count=1`>newestTag.txt'  
								TagName=readFile("newestTag.txt")
							}	
							//切换到tag对应的sourcecode
							sh "git checkout $TagName"
							sh 'versionline=`grep "version: " ./WebApp/manifest.yml`;echo ${versionline#*version: }>webAppVersion.txt'
							webAppVersion=readFile("webAppVersion.txt").tokenize()[0]
							sh 'versionline=`grep "version: " ./Worker/manifest.yml`;echo ${versionline#*version: }>dataWorkerVersion.txt'
							dataWorkerVersion=readFile("dataWorkerVersion.txt").tokenize()[0]
							sh 'versionline=`grep "version: " ./OTAWorker/manifest.yml`;echo ${versionline#*version: }>OTAWorkerVersion.txt'
							OTAWorkerVersion=readFile("OTAWorkerVersion.txt").tokenize()[0]
							sh 'chmod 777 gradlew'
							sh "./gradlew build"
							sh "./gradlew fatJar"
							sh "./gradlew makeManifest"
						}
					}
				}
			}
			
		}
		stage('RMMServer_Archive')
		{
			steps
			{
				script
				{
					container('jenkins-node-java')
					{
						webAppZipName="WebApp_${webAppVersion}.zip"
						dataWorkerZipName="DataWorker_${dataWorkerVersion}.zip"
						OTAWorkerZipName="OTAWorker_${OTAWorkerVersion}.zip"
						dir('RMMServer')
						{
							dir("WebApp_$webAppVersion")
							{
								sh 'cp ../WebApp/build/libs/* ./'
								sh "zip -r ${webAppZipName} ./*"
							}
							dir("DataWorker_$dataWorkerVersion")
							{
								sh 'cp ../Worker/build/libs/* ./'
								sh "zip -r ${dataWorkerZipName} ./*"
							}
							dir("OTAWorker_$OTAWorkerVersion")
							{
								sh 'cp ../OTAWorker/build/libs/* ./'
								sh "zip -r ${OTAWorkerZipName} ./*"
							}
							sh "ls"
							dir("Blob")
							{
								retry(3)
								{
									git credentialsId: "${git_credential}", url: 'http://advgitlab.eastasia.cloudapp.azure.com/WISE-PaaS_SRP_Pipeline/DevPublic.git'
									sh "python3 blobUploadDownload.py upload ${blob_container} webapp\\\\${webAppZipName}  ../WebApp_${webAppVersion}/${webAppZipName} ${blob_container_key}"
									sh "python3 blobUploadDownload.py upload ${blob_container} dataworker\\\\${dataWorkerZipName}  ../DataWorker_${dataWorkerVersion}/${dataWorkerZipName} ${blob_container_key}"
									sh "python3 blobUploadDownload.py upload ${blob_container} OTAWorker\\\\${OTAWorkerZipName}  ../OTAWorker_${OTAWorkerVersion}/${OTAWorkerZipName} ${blob_container_key}"
								}
								retry(3)
								{
									timeout(3)
									{
										//下载zip包
										sh "python3 ./blobUploadDownload.py download ${blob_container} webapp\\\\${webAppZipName}  ./${webAppZipName} ${blob_container_key}"
										sh "python3 ./blobUploadDownload.py download ${blob_container} dataworker\\\\${dataWorkerZipName}  ./${dataWorkerZipName} ${blob_container_key}"
										sh "python3 ./blobUploadDownload.py download ${blob_container} OTAWorker\\\\${OTAWorkerZipName}  ./${OTAWorkerZipName} ${blob_container_key}"
									}
								}
								if((!fileExists(webAppZipName)) || (!fileExists(dataWorkerZipName)))
								{
									error "package on blob does not exist!!!"
								}
							}
						}
					}
				}
			}
		}
	}
	post 
	{ 			
		failure 
		{ 
			script 
			{
						//1.build 失败
					container('jenkins-node-java')
					{
						echo "something wrong build fail........."
					}
			}
				
		}
		success
		{
			script
			{
				container('jenkins-node-java')
				{
					echo "SUCCUSS"
				}
			}
		}
	}
}