
# 如何改写部署App的pipeline #

本文档主要是介绍如何通过提供的部署App的pipeline模板，用简单、快捷的方法写出可以部署你自己的app到云上的步骤。

通过pipeline部署app总共分为以下几步：

1.	准备阶段
----------
准备阶段的工作：
- 获取你在apollo上配置的app信息
准备阶段又分为以下几个步骤：
	
- 从apollo上获取app的配置信息

	'''

		dir('json'){  
			//通过api获取apollo配置信息到appInfo.json
			sh 'curl "http://srplisting-apollo-wise-paas-dev.ali.wise-paas.com.cn/slsystem/v1/srps/${namespace}" -k > appInfo.json'
		}
		//获取python脚本用于处理从api获取到的json数据，并返回
		dir('cf_operation'){
			retry(2){
				git credentialsId: "${git_credential}", url: 'http://advgitlab.eastasia.cloudapp.azure.com/SRE/pipeline_all_config.git'
			}
		}
		sh 'python3 ./cf_operation/parseJson.py ./json/appInfo.json'
		//将处理过的python脚本读取到APPS列表中
		if(fileExists("config.json")){
			APPS = readJSON file: "config.json"
		}else{
			error "Connot get apollo config"
		}

	'''

- 初始化配置信息

	'''

		//将cf_org和cf_space转化为小写,在写manifest时使用
		def init(){
			cfOrgLow="${cf_org}".toLowerCase()
			cfSpaceLow="${cf_space}".toLowerCase()
			if(route_domain == ""){
				route_domain = "${cf_domain}"             
			}
			//获取要部署的app的个数
			appNums = APPS['param']['appNums']
		}

	'''

- 获取上一个版本

	'''
	
		//如果在smokeTest时失败需要回滚到上一个版本，此处提前获取上一个版本
		getPreviousVersion()

	'''	

- 获取org_id和space_id

	'''
		
		org_id=readFile('orgid.txt').trim()
		space_id=readFile('spaceid.txt').trim()


	'''

- 进行ELK检查
- 检查将要部署的app是否存在
	
	'''

		checkAppExist("beforeDeploy")

	'''

2. 部署阶段
----------

- 定义app绑定的服务

	'''
			
		//根据app要绑定的服务，来定义app的服务
		def defineService(appNames)

	'''

- 设置Manifest文件
	
	'''
	
		def setManifest(app,appZipName,serOrg,serviceMogo,servicePost,serviceRabbit)

	'''
	
	- 设置manifest分为以下两个步骤
		- 首先在blob上获取到你上传的*.zip包，并解压
			'''
			
				appZipName="${app['sourceZipName']}_${app['version']}.zip"
				sh "python3 ../cf_operation/blobUploadDownload.py download ${containerName} ${app['sourceZipPath']}\\\\${appZipName}  ./${appZipName}"
				sh "unzip -o -d ./ ${appZipName}"

			'''  
		- 然后就是更改manifest文件
			'''
				
				sh "cp manifest-production.yml manifest-${cfOrgLow}-${cfSpaceLow}.yml"
				if(LICENSE=="true"){
					sh "python3 ../cf_operation/ModifyManifest.py manifest-${cfOrgLow}-${cfSpaceLow}.yml ${cfOrgLow} ${cfSpaceLow} ${route_domain} ${app['namePrefix']} ${app['version']} ${servicePost} ${serviceMogo} ${serviceRabbit} ${serOrg},LICENSE:${space_id_md5}"	
				}else{
					sh "python3 ../cf_operation/ModifyManifest.py manifest-${cfOrgLow}-${cfSpaceLow}.yml ${cfOrgLow} ${cfSpaceLow} ${route_domain} ${app['namePrefix']} ${app['version']} ${servicePost} ${serviceMogo} ${serviceRabbit} ${serOrg}"	
				}

			'''

- 部署app
	
	'''
		
		//确定是否包含influxdb
		if(app.containsKey('influxdb')){
		   sh "cf bind-service ${app['name']}-${app['version']} ${app['influxdb_service_instance_name']}"
		}
		if(IfELK=="true"){
			sh "cf bind-service ${app['name']}-${app['version']} ELK"
		}
		if(sso_url==""){
			sh "cf set-env ${app['name']}-${app['version']} sso_url ''"
	
		 }else{
			sh "cf set-env ${app['name']}-${app['version']} sso_url ${sso_url}"
		} 
		if(mp_url==""){
			sh "cf set-env ${app['name']}-${app['version']} mp_url ''"
	
		 }else{
			sh "cf set-env ${app['name']}-${app['version']} mp_url ${mp_url}"
		}		
		//如果app_seq不为空，那么久按顺序启动			
		if(app["app_seq"] == null){
			sh "cf start ${app['name']}-${app['version']}"
		}

	'''

3. 顺序启动阶段
----------
- 根据'app_seq'按顺序启动app
	
	'''
		
		for(int i=0;i<appNums;i++){
			 "${k}").toString()
			sKey(appM)){
			eqMap.get("1")
			".toString()
			eqMap(mmm)
			eqMap.get("${n}")

			fullAppName = startSeqMap.get("${n}")
			echo fullAppName
			sh "cf start ${fullAppName}"
		}
		k = k+1
		n = n+1

	'''

4. 检查app是否部署成功
----------
- 此处是检查app是否部署成功，即:部署完成后状态是否是started的

	'''
	
		for(int i = 0;i<appNums;i++){
			appM = ("app" + "${m}").toString()
			if(APPS.containsKey(appM) && APPS[appM]['status']=="0"){
				checkedApp = APPS[appM]
				checkedStatus = checkAppExist(checkedApp)
				APPS[appM]['checkStatus'] = checkedStatus
				if(checkedAppStatus == "0"){
					rollStatus = true
				}
			}
			m = m+1
		}
		echo "checkApp end .............."

	'''

5. 回滚
----------

- 如果上面的检查有状态是failed或者不存在，那么就会进行回滚。即：删除刚部署的所有app
	
	'''
		
		for(int i=0;i<appNums;i++){
			appM = ("app" + "${m}").toString()
			if(APPS.containsKey(appM)==true && APPS[appM] != null){
				if(APPS[appM]['status'] == "0"){
					dApp = APPS[appM]['name']
					dVersion = APPS[appM]['version']
					dAppName = "${dApp}-${dVersion}"
					echo dAppName
					sh "cf delete ${dAppName} -r -f"
				}
			}
			m = m+1
		}
		echo "deploy app failed,it's will roll back to last version"
		return

	'''

6. 给app_host赋值
----------

- app_host:主要用于绑定URL时使用

	'''
		
		appNamePrefix = APPS[appM]['namePrefix']
		if(cfOrgLow=="wise-paas" && cfSpaceLow=="srp"){
			app_host="${appNamePrefix}"
		}else if(cfOrgLow=="wise-paas-dev" && cfSpaceLow=="srp"){
			app_host="${appNamePrefix}"
		}
		else if(cfOrgLow=="wise-paas-stage" && cfSpaceLow=="srp"){
			app_host="${appNamePrefix}"
		}else{
			app_host="${appNamePrefix}"
		}
		
	'''

7. 蓝绿部署
----------

- 蓝绿部署是将上上一个版本（如果有）删除，并将上一个版本停止，即：使其状态处于stopped（这是防止这个版本没有通过smokeTest,返回到上一个版本）,该stage是处于并行状态的

'''

	parallel {
		stage("bluegreenswitch_app1"){
			steps{
				script{
					app00 = APPS["app0"]
					bluegreenswitch(app00)
				}
			}
		}
		stage('bluegreenswitch_app2'){
			steps{
				script{
					app01 = APPS['app1']
					bluegreenswitch(app01)
				}
			}
		}
		stage('bluegreenswitch_app3'){
			steps{
				script{
					app02 = APPS['app2']
					bluegreenswitch(app02)
				}
			}
		}
	}

'''

8. smoke test(冒烟测试)
----------

- smoke test是调用部署上去的app接口，然后检查其response状态是不是200，来判断这个app是否可以正常访问

	'''
	
		for(int i=0;i<appNums;i++){
			appM = ("app" + "${m}").toString()
			if(APPS[appM].containsKey("${appM}")){
				if(APPS[appM]['isFronted'] == "1"){
					smokeTest("${APPS[appM]['name']}-${APPS[appM]['version']}")
				}
			}
			m = m+1
		}

	'''

9. 异常处理（post机制）
----------

- failure：如果在以上各个阶段出现问题都会进入到这个failure代码块中，该区块代码主要是在以上阶段出错后，完成以下三步：
	- 删除刚部署了的app（如果没部署，就不会删除）
	- 然后到上一个版本（如果上一个版本没有进行stopped操作，就会跳过）
	- 报出error信息

- success：当代码顺利执行完毕后就会进入该代码块中