# SRP上架需要符合的规范 #
本文档主要给出了SRP在上架时需要符合的规范。

- 凡是标注**[必须]**，是SRP上架时必须满足的规范
- 凡是标注[建议]，是SRP上架时建议满足的规范
- 凡是标注[Level 1]，是SRP提出上架时，Listing Management团队检查的规范

## 1、SSO Integration ##
#### 1.1 **[Level 1]** SSO登录页面 ####
- **[必须]** 登录页面样式：整合规范的CSS Template 
- [建议] SSO登录页面支持忘记密码
- [建议] 用户登陆后，可以正确记住用户名

#### 1.2 SRP Token的产生方式 ####
- **[必须]** SRP Token产生方式：base64Url\_encode(AES_encode(timestamp-srpName)) (timestamp为当下时间）

#### 1.3 **[Level 1]** SSO URL ####
- **[必须]** 二者择一即可。
    - 环境变量中有单独设置SSO URL，优先以环境变量中设置的SSO URL进行连接 
    - 环境变量中未单独设置SSO URL  
  		根据不同的org自动选择不同的SSO：  
  		（1）WISE-PaaS org： https://portal-sso.${domain}  
  		（2）WISE-PaaS-Stage org：https://portal-sso-stage.${domain}  
  		（3）WISE-PaaS-Dev org：https://portal-sso-develop.${domain}  
  		（4）其他org：https://portal-sso.${domain}  


#### 1.4 **[Level 1]** SRP默认权限支持 ####
**[必须]** 各账户SRP默认权限如下：  

- admin帐户：不做要求
- tenant帐户：拥有SRP所在org的情况下，默认无需特别授权即可访问；默认拥有最高SRP最高权限
- developer帐户：拥有SRP所在org的情况下，建议无需特别授权即可访问；默认拥有SRP最低权限，需要时可提权
- srpUser帐户：被授权后，可正常访问SRP

#### 1.5 **[Level 1]** 整合SSO最新版本的API ####
- **[必须]** 整合前置最新版号（目前为/v2.0）的SSO API端点

#### 1.6 正确的app id ####
- [建议] SRP于SSO注册时填入SRP于CF上的app id
    - SRP注册  

		POST /v2.0/srps  
		{  
			 "name": "<string>",  
			 "orgId": "<string>",  
			 "spaceId": "<string>",  
			 "appId": "<string>",  
			 "scopes": []  
		}  
	    **Authentication  
	    SRPToken in HTTP header  
	    X-Auth-SRPToken: <SRPToken>
    - SRP升级  
	
		PUT /v2.0/srps  
		{  
		  "name": "<string>",  
		  "orgId": "<string>",  
		  "spaceId": "<string>",  
		  "appId": "<string>",  
		  "scopes": []  
		}  
	    **Authentication  
	    SRPToken in HTTP header  
	    X-Auth-SRPToken: <SRPToken>

#### 1.7 Token过期处理 ####
- **[必须]** 使用refresh token API换发token package，而非每次都透过/auth/native API取得新的token package

- [建议] 可于令牌过期前10分钟内重新换发令牌
- [建议] 检查令牌的时机：
    - 应用程式设计为自动登出模式，可于应用程式API被呼叫时，检查令牌之有效期限
    - 应用程式设计为非自动登出模式，应用程式可设置排程定期检查令牌之有效期限

#### 1.8 **[Level 1]** 多云支持 ####
- **[必须]** 请务必不能 hardcode SSO URL，同一套SRP可能被部署至不同Domain的WISE-PaaS云平台，相应的有不同的SSO URL

## 2、DB ##
#### 2.1 Service重连机制 ####
- **[必须]** Service断线后，APP在Service恢复后可自动重连
- [建议] APP重连间隔采用二进制退避算法控制重连间隔，避免在Service异常期间，对Service造成更大负荷
- **[必须]** 账号密码错误不需要再进行Retry，容易造成Service负担
- [保留，以后讨论] 因Consumption management机制被切断或限制权限的错误处理（Reserved）（保留，以后讨论）

#### 2.2 Service Credentials ####
- **[必须]** Service Name支持以前缀匹配，如MongoDB Service Name可能为mongodb，或为Mongodb-shard

#### 2.3 Connection ####
- **[必须]** 建立并重复利用Connection pool，一个APP不超过10条连线
- **[必须]** 使用完毕后Connection务必关闭
- [建议] Idle Timeout建议不超过10分钟
- **[必须]** 支持单节点与多节点连线机制，可依Credential内容动态切换

#### 2.4 建立Index ####
- **[必须]** 应建立适当的index，提升query效能
- **[必须]** APP升级变动DB schema时，owner要与前版本相同

## 3、IoTHUB ##
#### 3.1 Publisher（Producer） ####
##### 3.1.1 重连机制 #####
- **[必须]** Rabbitmq Service断线恢复后，Producer端需自动重连，建议重连间隔采用二进制退避算法控制重连间隔，避免在Service异常期间，对Service造成更大负荷
- **[必须]** 账号密码错误就不需再Retry，若有需要Retry必须加sleep time控制重连间隔

##### 3.1.2 Connection #####
- **[必须]** 使用完毕的Connection务必关闭（必须）
- **[必须]** 若Producer端是发送完毕后进入休眠模式，请确保关闭与Rabbitmq Service的Connection
- **[必须]** 减少建立Connection的数量与次数， 程式启动后使用同一个Connection来推送资料
- [建议] 需支援DCCS规范：  
    - 确保使用金轮来兑换连线帐密   
    - 连不上Rabbitmq Service先询问DCCS  
    - 依据DCCS回应状态码进行Retry或停止重连  
    - Retry是需要采用随机延迟时间来控制Retry频率 

##### 3.1.3 Publish Data #####
- [建议] 设置合理（配合Consumer的能力）的发送频率与封包大小

##### 3.1.4 High Avaliability #####
- [建议] 使用QoS 1以确保讯息正确发送（Ack）（与效能互斥）
- [建议] Producer端资料无法向Rabbitmq Service投递时需要在本地缓存

##### 3.1.5 High Performance #####
- [建议] 使用QoS 0（与高可用互斥）

#### 3.2 Subscriber（Consumer） ####
##### 3.2.1 Credentials #####
- [建议] 使用绑定的方式取得Credential

##### 3.2.2 Connection #####
- **[必须]** 减少建立Connection的数量与次数，程式启动后使用同一个Connection来接收资料
- [建议] 尽可能使用相同的Client ID
- **[必须]** Timeout要小于Keep_Alive时间（否则Consumer会定期从Broker断开）

##### 3.2.3 重连机制 #####
- **[必须]** Consumer端侦测到与Rabbitmq Service之间连线断开时，需自动重连

##### 3.2.4 High Avaliability #####
- **[必须]** Consumer端与Rabbitmq Service断线重连后，处理大量IoTHub queued message时不能Crash（缓横处理）
- [建议] 使用Durable Queue及Mirror Queue（与高效能互斥）

##### 3.2.5 High Performance #####
- [建议] 不采用Durable Queue（与高可用互斥）
- [建议] 使用自动ACK机制（与高可用互斥）

## 4、APP ##
#### 4.1 APP 12 Factors ####
- [建议] APP按照SaaS软件开发的12要素，并结合CloudFoundry的特性，来进行SaaS软件的开发，增加App的可靠性和健壮性，详细内容请参考：  
链接：[https://portal-technical.wise-paas.com/doc/document-portal.html#CloudFoundryCli-2](https://portal-technical.wise-paas.com/doc/document-portal.html#CloudFoundryCli-2)

#### 4.2 WISE-PaaS BuildPack BuildPack Spec规范 ####
- **[必须]** Buildpacks为Cloud Foundry上的应用程序提供了框架和运行时支持。Buildpacks通常会检查你的应用程序，以确定要下载哪些依赖项，以及如何配置应用程序来与绑定的服务进行通信。当你推送一个应用程序时，Cloud Foundry会自动检测到一个合适的BuildPack。这个buildpack用于编译或运行你的应用程序。BuildPack规范请参考：  
链接：[https://portal-technical.wise-paas.com/doc/document-portal.html#CloudFoundryCli-3](https://portal-technical.wise-paas.com/doc/document-portal.html#CloudFoundryCli-3)

#### 4.3 **[Level 1]** App Manifest&Name&Route 规范 ####
##### 4.3.1 APP Manifest规范 #####
- **[必须]** APP Manifest规范请参考:   
  
		applications:   
		- name: portal-javasamlecode-1.0.0       
		  memory: 512M    
		  disk_quota: 1024M    
		  instances: 1 
		  buildpack: Java_v3-19-2 
		  health-check-type: http 
		  services:      
		  - mongodb      
		  - rabbitmq 
    - 说明：
  		（1）app健康检查的类型，有port, process, http三种类型，如果不指定，默认为port，一般web服务类的app使用健康检查类型为http，比如portal或者api类的app，backgroud类型的app使用健康检查类型为process，比如worker类的app，详细使用情况请参考[doc](https://docs.pivotal.io/pivotalcf/2-2/devguide/deploy-apps/healthchecks.html)。
  		（2）name需要遵循的规范请参考App Name规范。
  		（3）因为host中需要添加org和space信息，所以会在部署的pipeline中根据App Route规范自动去添加host参数。
  		（4）buildpack需要遵循的规范请参考上文4.2节。
  		（5）如果不依赖任何service，就不需要services选项，如果依赖postgresql，不要写在manifest中，因为bind postgresql的时候需要指定group，要用cli命令去bind，直接写在manifest中不能指定group。
  		（6）环境变量env中的有些参数是根据部署时的情况动态变化的，比如org_name，org_id，mongodb_service_name，postgresql_service_name，rabbitmq_service_name，所以需要在自动部署的pipeline中动态修改。

##### 4.3.2 APP Route规范 #####
- **[必须]** 自动部署的App（SRP），为了避免Route冲突，会在Route中加上orgname和spacename进行区分，比如在一个名为AdvIIOT-EnE的org，名为ECU的space，Route规范如下：

    - User Interface 

		spec: portal-<project>-orgname-spacename  
		example: portal-scada-adviiot-ene-ecu  
    - API Interface  

	    spec: api-<project>-orgname-spacename  
		example: api-scada-adviiot-ene-ecu
    - Document  

		spec: docs-<project>-orgname-spacename  
  		example: docs-scada-adviiot-ene-ecu 
    - Dashboard  

		spec: dashboard-<project>-orgname-spacename  
  		example: dashboard-scada-adviiot-ene-ecu 

##### 4.3.3 APP Name规范 #####
- **[必须]** App的名字由app类型+项目名称+版本号组成，版本号由三位组成，一般从1.0.0开始。  

    - User Interface  

	    spec: portal-<project>-<version>  
  	    example: portal-scada-1.0.0
    - API Interface 

		spec: api-<project>-<version>  
  		example: api-scada-1.0.0  
    - BackGround App 

	    spec: <project>-worker-<version>  
        example: scada-worker-1.0.0 
    - Document 

		spec: docs-<project>-<version>  
  		example: docs-scada-1.0.0  
    - Dashboard

		spec: dashboard-<project>-<version>  
  		example: dashboard-scada-1.0.0 

#### 4.4 **[Level 1]** Minimum Mem Quota ####
- **[必须]** SRP上架必须给出App所需要的最小内存

## 5、Dashboard Integration ##
#### 5.1 SRP的dashboard，SRP-Frame管理 ####
- **[必须]** 不同的SRP分别在不同的dashboard org中进行管理，不同org中，用户，datasource，dashboard相互隔离。部署时先调用api创建SRP 同名的org，然后调用api导入datasource，导入dashboard，导入SRP-Frame 到刚才创建的org中。
- **[必须]** SRP 的dashboard可统一导出成json文件管理，在部署时通过API导入到目标Dashboard App中。
- **[必须]** SRP-Frame的配置可统一导出成json文件管理，在部署时通过API导入到目标Dashboard App中。

#### 5.2 SRP-Frame的注意事项 ####
- **[必须]** 使用Dashboard中整合的SRP-Frame，不要使用独立app的旧版，旧版已经不维护。
- **[必须]**  SRP-Frame中配置的dashboard连接建议就用当前所在的dashboard中的页面。
- [建议] SRP-Frame 中dashboard链接的配置建议用/d/uid/dashboard slug，可从dashboard url中获取到。防止使用/dashboard/db/dashboard slug，如果dashboard在不同folder下同名的时候SRP-Frame加载不到唯一正确的页面。 
  		
        example：{"title":"SCADA","type":"dashboard","dashboard":"/d/DrmDnRwmk/plugin-upload"}
