## SRP上架需要提供的参数 ##
每个SRP上架时，需要提供一组参数，以下针对各参数进行解释说明，并以RMM为例给出了示例。  
### 1、参数说明 ###
##### 1.1 version #####

- 说明：即SRP的大版本号，注意不是APP的版本号。
- 示例：以RMM为例，输入格式如下。
	    
	    version = 1.0.60

##### 1.2 extra_param #####

- 说明：即每个SRP需要用到的附加参数可以写在这里，该项是每个SRP独有的，如果没有，则为空。
- 示例：比如RMM使用时需要用到RMMEIAgent，这里就提供了RMMEIAgent的地址和版本号，输入格式如下：

		extra_param = 
						{  
    						"AgentDownloadSite": "https://advantechfiles.blob.core.windows.net/wise-paas-marketplace/Download/RMM/RMMEIAgent_V1.0.11.zip",  
    						"AgentVersion": "1.0.11"  
						}

##### 1.3 infra_services #####

- 说明：即SRP所要绑定的服务。
- 示例：比如RMM需要绑定MongoDB、Rabbitmq、Postgresql服务，输入格式如下：  


		infra_services = 
						[
					        {  
					        	"name": "MongoDB",  
					        	"version": "",  
					        	"discription": "" 
					        },  
					        {  
					        	"name": "Rabbitmq",  
					        	"version": "",  
					        	"discription": ""
					        },
					        {  
					        	"name": "Postgresql",  
					        	"version": "",  
					        	"discription": ""
					        }
				        ]         

其中：  

- name：服务的名称；  
- version：服务的版本，如果没有，则输入为空；  
- discription：对该服务的描述，可以输入为空。

##### 1.4 apps #####

- 说明：即每个SRP中包括的APP，部署时需要提供每个APP的相应参数，参数说明如下：
  1. name：APP的名字；
  2. version：APP的版本号；
  3. memory：APP所需要的运行内存；
  4. infra_services：APP所要绑定的服务，其中Key为Service Name，Value为Service的一些参数；
  5. params：SRP Pipeline中需要的参数，必填的有三项：sourceZipName、sourceZipPath、urlPrefix，若SRP Pipeline还需要其他参数，也可在这里填写，比如RMM Pipeline需要提供SERVER_ACTIVATENAME，就在这里填写。其中：   
  （1）sourceZipName：blob上APP打包名称；  
  （2）sourceZipPath：blob上APP打包地址；  
  （3）urlPrefix：APP部署如果要绑定URL，则需要提供urlPrefix（如RMM的APP，portal-rmm）；
  6. app_seq：APP启动顺序。如果APP之间相互依赖，需要指定APP启动顺序，否则可以不用指定启动顺序，APP为并行启动。
- 示例：以RMM为例，RMM包括3个APP，3个APP部署时所需参数输入格式如下：  
		
		apps = 
				[
		        	{  
			        	"name": "ota-worker",  
			        	"version": "1.0.38",  
			        	"memory": "1024",  
			        	"infra_services":   
			        	{  
			            	"mongodb": "",    
			            	"rabbitmq": "",  
			           		"postgresql":   
			             	{  
			                	"postgresql_service_group": "g_rmm,g_ota,g_rw_public"    
			             	}  
			        	},  
			        	"params":   
			        	{   
			            	"sourceZipName": "OTAWorker",  
			            	"sourceZipPath": "OTAWorker",  
			            	"urlPrefix": "",  
			            	"SERVER_ACTIVATENAME": "EdgeSense_EnSaaS"  
			        	},  
			        	"app_seq": "3"
					},
					{  
			        	"name": "portal-rmm",  
			        	"version": "1.0.93",  
			        	"memory": "512",  
			        	"infra_services":  
			         	{  
			            	"mongodb": "",  
			            	"rabbitmq": "",  
			            	"postgresql":   
			             	{  
			                	"postgresql_service_group": "g_rmm,g_ota,g_rw_public"  
			             	}  
			        	},  
			        	"params":   
						{  
			            	"urlPrefix": "portal-rmm,api-rmm",      
			            	"sourceZipName": "WebApp",  
			            	"sourceZipPath": "webapp",   
			            	"SERVER_ACTIVATENAME": "EdgeSense_EnSaaS"  
			        	},  
			        	"app_seq": "2"
					},
					{  
			        	"name": "rmm-worker",  
			        	"version": "1.0.93",  
			        	"memory": "768",  
			        	"infra_services":  
			         	{  
			           		"mongodb": "",  
			            	"rabbitmq": "",  
			            	"postgresql":   
			            	{  
			                	"postgresql_service_group": "g_rw_public,g_rmm"  
			            	}  
			        	},  
			        	"params":   
			        	{   
			            	"sourceZipName": "DataWorker",  
			            	"sourceZipPath": "dataworker",  
			            	"urlPrefix": "",  
			            	"SERVER_ACTIVATENAME": "EdgeSense_EnSaaS"   
			        	},  
			        	"app_seq": "1"
					}
				]  

##### 1.5 app_service #####

- 说明：即SRP所依赖的其他SRP。其中：  
  1. name：所依赖的SPR的名称；  
  2. version：所依赖SRP的版本号。 
- 示例：比如，RMM不依赖于其他SRP，所以这项为空。APM依赖于RMM、Dashboard等，因此APM这一项会填写RMM、Dashboard，APM的该项输入格式如下。  
		
		app_service = 
						[
							{  
								"name": "RMM",
								"version": "<string>"  
							},
							{  
								"name": "Dashboard",
								"version": "<string>"
							}
						]


##### 1.6 srp_name #####

- 说明：即SRP的名称。
- 示例：以RMM为例，srp_name为RMM，输入格式如下。
        
        srp_name = RMM

##### 1.7 catalog #####

- 说明：SRP归属类别。
- 示例：以RMM为例，RMM属于Device Management，此项就填写Devcie Management，输入格式如下。
        
        catalog = Device Management

##### 1.8 description #####

- 说明：即针对SRP用途所做的描述。
- 示例：以RMM为例，description为IoT Device Remote Monitoring and Management Platform，输入格式如下。
    
        description = IoT Device Remote Monitoring and Management Platform

##### 1.9 release_note #####

- 说明：发布说明，输入格式如下。

		release_note = 
						[Important Notice] February 19 WISE-PaaS/EdgeSense v-1.0.58 Release
				
						Dear WISE-PaaS Users:
						
						WISE-PaaS/EdgeSense v-1.0.58 Release
						
						[EdgeSense Release Note]
						Release Package Version: v-1.0.48 -> v-1.0.58
						
						### Core Component:
						- rmm-worker: v-1.0.96
						- ota-worker: 1.0.40
						- rmm-portal: 1.0.96
						### New Features:
						- Support Azure storage for different area (Global Azure, China, Germany, US Government)
						- Support SAS (Shared Access Signature) to access Azure blob container.
						- Release RMM Agent v-1.1.5, download installer from EdgeSense portal or here.
						### Enhancement:
						- [Functionality] Provide user interface to subscribe/unsubscribe notification.
						- [Functionality] Support native authentication (anonymous users) for WISE-PaaS Dashboard to access.
						- [Functionality] Support minimum report interval for adjusting.
						- [Functionality] Show unit and min/max for each sensor type.
						- [Functionality] Update user manual and Get started.
						
						
						Best Regards
						
						WISE-PaaS@HongKong
						
						[WISE-PaaS Knowledge Portal](http://select.advantech.com/knowledge-portal/)
						
						[WISE-PaaS Forum](https://wiseqa.wise-paas.com/)
						
						[ Technical Portal](https://portal-technical.wise-paas.com/)
						
						[ Support System ](https://portal-support.wise-paas.com)

##### 1.10 type #####

- 说明：该项均填写为srp，输入格式如下。
	
	    type = srp

### 2、提供参数格式 ###
以RMM为例，SRP上架时，请将第1节中的参数以下列形式提供给我们

<table class="tg">
 <tr>
      <th align="center">Key</th>
      <th align="center">Value</th>
    </tr>
    <tr>
      <td>version</td>
      <td>1.0.60</td>
    </tr>
    <tr>
      <td>extra_param</td>
      <td>{<br>　　"AgentDownloadSite":"https://advantechfiles.blob.core.windows.net/wise-paas-marketplace/Download/RMM RMMEIAgent_V1.0.11.zip",<br>　　"AgentVersion":"1.0.11"<br>}</td>
    </tr>
    <tr>
      <td>infra_services</td>
      <td>[<br>　　{<br>　　　　"name": "MongoDB",<br>　　　　"version": "",<br>　　　　"discription": ""<br>　　},     <br>　　{<br>　　　　"name": "Rabbitmq",<br>　　　　"version": "",<br>　　　　"discription": ""<br>　　},<br>　　{<br>　　　　"name": "Postgresql",<br>　　　　"version": "",<br>　　　　"discription": ""<br>　　}<br>]</td>
    </tr>
    <tr>
      <td>apps</td>
      <td>[<br>　　{<br>　　　　"name": "ota-worker",<br>　　　　"version": "1.0.38",<br>　　　　"memory": "1024",<br>　　　　"infra_services":<br>　　　　{<br>　　　　　　"mongodb": "",<br>　　　　　　"rabbitmq": "",<br>　　　　　　"postgresql":<br>　　　　　　{<br>　　　　　　　　"postgresql_service_group": "g_rmm,g_ota,g_rw_public"<br>　　　　　　}<br>　　　　},<br>　　　　"params":<br>　　　　{<br>　　　　　　"sourceZipName": "OTAWorker",<br>　　　　　　"sourceZipPath": "OTAWorker",<br>　　　　　　"urlPrefix": "",<br>　　　　　　"SERVER_ACTIVATENAME": "EdgeSense_EnSaaS"<br>　　　　},<br>　　　　"app_seq": "3"<br>　　},<br>　　{<br>　　　　"name": "portal-rmm",<br>　　　　"version": "1.0.93",<br>　　　　"memory": "512",<br>　　　　"infra_services"<br>　　　　{<br>　　　　　　"mongodb": "",<br>　　　　　　"rabbitmq": "",<br>　　　　　　"postgresql":<br>　　　　　　{<br>　　　　　　　　"postgresql_service_group": "g_rmm,g_ota,g_rw_public"<br>　　　　　　}<br>　　　　},<br>　　　　"params"<br>　　　　{<br>　　　　　　"urlPrefix": "portal-rmm,api-rmm",<br>　　　　　　"sourceZipName": "WebApp",<br>　　　　　　"sourceZipPath": "webapp",<br>　　　　　　"SERVER_ACTIVATENAME": "EdgeSense_EnSaaS"<br>　　　　},<br>　　　　"app_seq": "2"<br>　　},<br>　　{<br>　　　　"name": "rmm-worker",<br>　　　　"version": "1.0.93",<br>　　　　"memory": "768",<br>　　　　"infra_services":<br>　　　　{<br>　　　　　　"mongodb": "",<br>　　　　　　"rabbitmq": "",<br>　　　　　　"postgresql":<br>　　　　　　{<br>　　　　　　　　"postgresql_service_group": "g_rw_public,g_rmm"<br>　　　　　　}<br>　　　　},<br　　　　"params":<br>　　　　{<br>　　　　　　"sourceZipName": "DataWorker",<br>　　　　　　"sourceZipPath": "dataworker",<br>　　　　　　"urlPrefix": "",<br>　　　　　　"SERVER_ACTIVATENAME": "EdgeSense_EnSaaS"<br>　　　　},<br>　　　　"app_seq": "1"<br>　　}<br>]</td>
    </tr
    <tr>
      <td>app_services</td>     
	  <td>[<br>　　{<br>　　　　"name": "",<br>　　　　"version": ""<br>　　},<br>　　{<br>　　　　"name": "",<br>　　　　"version": ""<br>　　}<br>]</td>
    </tr>
	<tr>
      <td>srp_name</td>
      <td>RMM</td>
    </tr>
	<tr>
      <td>catalog</td>
      <td>Device Management</td>
    </tr>
	<tr>
      <td>description</td>
      <td>IoT Device Remote Monitoring and Management Platform</td>
    </tr>
	<tr>
      <td>release_note</td>
      <td>[Important Notice] February 19 WISE-PaaS/EdgeSense v-1.0.58 Release  <br>Dear WISE-PaaS Users:  <br>WISE-PaaS/EdgeSense v-1.0.58 Release  <br>[EdgeSense Release Note] <br>Release Package Version: v-1.0.48 -> v-1.0.58  <br>### Core Component: <br>- rmm-worker: v-1.0.96 <br>- ota-worker: 1.0.40 <br>- rmm-portal: 1.0.96  <br>### New Features: <br>- Support Azure storage for different area (Global Azure, China, Germany, US Government) <br>- Support SAS (Shared Access Signature) to access Azure blob container. <br>- Release RMM Agent v-1.1.5, download installer from EdgeSense portal or here. <br>### Enhancement: <br>- [Functionality] Provide user interface to subscribe/unsubscribe notification. <br>- [Functionality] Support native authentication (anonymous users) for WISE-PaaS Dashboard to access. <br>- [Functionality] Support minimum report interval for adjusting. <br>- [Functionality] Show unit and min/max for each sensor type. - [Functionality] Update user manual and Get started.   <br>Best Regards <br>WISE-PaaS@HongKong  [WISE-PaaS Knowledge Portal](http://select.advantech.com/knowledge-portal/)  <br>[WISE-PaaS Forum](https://wiseqa.wise-paas.com/)  <br>[ Technical Portal](https://portal-technical.wise-paas.com/)  <br>[ Support System ](https://portal-support.wise-paas.com)</td>
    </tr>
	<tr>
      <td>type</td>
      <td>srp</td>
    </tr>
</table>




